<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Form\Type\Admin;

use Sylius\Bundle\ChannelBundle\Form\Type\ChannelChoiceType;
use Sylius\Bundle\MoneyBundle\Form\Type\MoneyType;
use Sylius\Bundle\ResourceBundle\Form\Type\AbstractResourceType;
use Sylius\Bundle\ResourceBundle\Form\Type\ResourceAutocompleteChoiceType;
use Sylius\Component\Channel\Repository\ChannelRepositoryInterface;
use Sylius\Component\Core\Context\ShopperContextInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Symfony\Component\Form\CallbackTransformer;
use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\OptionsResolver\Options;
use Symfony\Component\OptionsResolver\OptionsResolver;

class ChannelPricingType extends AbstractResourceType
{

    public function __construct(
        string                                      $dataClass,
        array                                       $validationGroups,
        private readonly ShopperContextInterface    $shopperContext,
        private readonly RepositoryInterface        $channelPricingRepository,
        private readonly ChannelRepositoryInterface $channelRepository,
        private readonly DataTransformerInterface   $channelPricingConverterTransformer
    )
    {
        parent::__construct($dataClass, $validationGroups);
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        /** @var ChannelInterface $channel */
        $channel = $this->shopperContext->getChannel();

        $baseCurrency = $channel->getBaseCurrency();
        $currencyCode = null === $baseCurrency ? $this->shopperContext->getCurrencyCode() : $baseCurrency->getCode();

        $builder->addModelTransformer($this->channelPricingConverterTransformer);

        $builder
            ->add('channelCode', ChannelChoiceType::class, [
                'label' => 'sylius.ui.channel',
                'required' => true,
            ])
            ->get('channelCode')->addModelTransformer(new CallbackTransformer(
                fn(?string $channelCode): ?ChannelInterface => $channelCode !== null ? $this->channelRepository->findOneBy(['code' => $channelCode]) : null,
                fn(?ChannelInterface $channel): string => $channel !== null ? $channel->getCode() : '',
            ));

        $builder
            ->add('productVariant', ResourceAutocompleteChoiceType::class, [
                'resource' => 'sylius.product_variant',
                'choice_name' => 'descriptor',
                'choice_value' => 'code',
            ])
            ->add('price', MoneyType::class, [
                'label' => 'sylius.ui.price',
                'currency' => $currencyCode,
            ])
            ->add('originalPrice', MoneyType::class, [
                'label' => 'sylius.ui.original_price',
                'required' => false,
                'currency' => $currencyCode,
            ])
            ->add('minimumPrice', MoneyType::class, [
                'label' => 'sylius.ui.minimum_price',
                'required' => false,
                'currency' => $currencyCode,
            ])
        ;

        $builder->addEventListener(FormEvents::SUBMIT, function (FormEvent $event) use ($options): void {
            $channelPricing = $event->getData();

            if (!$channelPricing instanceof $this->dataClass || !$channelPricing instanceof ChannelPricingInterface) {
                $event->setData(null);

                return;
            }

            if ($channelPricing->getPrice() === null && $channelPricing->getOriginalPrice() === null) {
                $event->setData(null);

                if ($channelPricing->getId() !== null) {
                    $this->channelPricingRepository->remove($channelPricing);
                }

                return;
            }

            $event->setData($channelPricing);
        });
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        parent::configureOptions($resolver);

        $resolver
            ->setDefaults([
                'label' => fn(Options $options): string => $this->shopperContext->getChannel()->getName(),
            ]);
    }
}
